#!/usr/bin/env python
# -*- coding: utf-8 -*-

#####################################################################################
## This code is released under the GPL liscence
## http://www.gnu.org/licenses/gpl.html
## Copyright 2016 By Geoff Weiss
## Based on nds_export.py by M-A Loyer (weirdfox--gmail.com) and
## information from http://shallowsky.com by Akkana Peck
#####################################################################################
## The Gimp exporter for Apple IIgs Images
#####################################################################################

import struct
import gimp
from gimpfu import *
from array import array

gettext.install("gimp20-python", gimp.locale_directory, unicode=True)

## Convert a rgb color to IIgs pallete GGGGBBBB0000RRRR format
def rgb32toint16(rr,gg,bb):
    color = (bb>>4) | ((gg>>4)<<4) | ((rr>>4)<<8)
    return color

## Check to to see palette is 640 mode dithered
def checkDitherPalette(colormap):

    return (colormap == (0, 0, 0, 0, 0, 127, 127, 127, 0, 127, 127, 127, 127, 0, 0, 127, 0, 127, 255, 127, 0, 255, 127, 127, 0, 127, 0, 0, 127, 127, 127, 255, 0, 127, 255, 127, 127, 127, 127, 127, 127, 255, 255, 255, 127, 255, 255, 255))

## Write the palette from colormap to fileOut in 320 mode format
def createPalette320(colormap,nbColors,fileOut):
    # Write the palette to the file
    for i in range(0,nbColors):
        try:
            color = rgb32toint16(colormap[i*3+0],colormap[i*3+1],colormap[i*3+2])
        except IndexError:
            color = 0
            if (i == (nbColors - 1)) and ((color0 >> 12) | ((color0 & 0x0f00) >> 8) | ((color0 & 0x00f0) >> 4) < 8):
               color = 0xfff0

        if i == 0:
            color0 = color

        fileOut.write(struct.pack("<H", color))
        gimp.progress_update(float(i*(0.2/nbColors) + 0.8))

## Write the palette from colormap to fileOut in 640 mode format
def createPalette640(colormap,nbColors,fileOut):
    # Write the palette to the file
    for h in range(0,4):
     for i in range(0,nbColors):
        try:
            color = rgb32toint16(colormap[i*3+0],colormap[i*3+1],colormap[i*3+2])
        except IndexError:
            color = 0
        fileOut.write(struct.pack("<H", color))
        gimp.progress_update(float(i*(0.2/nbColors) + 0.8))


## Write the palette from colormap to Desktop dither 640 mode
def createPalette640Dither(fileOut):
    # Write the palette to the file
    for h in range(0,2):
        color = rgb32toint16(0,0,0)
        fileOut.write(struct.pack("<H", color))
        color = rgb32toint16(255,0,0)
        fileOut.write(struct.pack("<H", color))
        color = rgb32toint16(0,0,255)
        fileOut.write(struct.pack("<H", color))
        color = rgb32toint16(255,255,255)
        fileOut.write(struct.pack("<H", color))
        color = rgb32toint16(0,0,0)
        fileOut.write(struct.pack("<H", color))
        color = rgb32toint16(0,255,0)
        fileOut.write(struct.pack("<H", color))
        color = rgb32toint16(255,255,0)
        fileOut.write(struct.pack("<H", color))
        color = rgb32toint16(255,255,255)
        fileOut.write(struct.pack("<H", color))
        gimp.progress_update(float(h*(0.2/2) + 0.8))


def index_copy_of_image(img,index):
    pdb.gimp_edit_copy_visible(img)
    imageIndex = pdb.gimp_edit_paste_as_new()
    pdb.gimp_image_flatten(imageIndex)
    pdb.gimp_image_convert_indexed(imageIndex,0,1,index,FALSE,FALSE,"")
    return (imageIndex,pdb.gimp_image_get_active_layer(imageIndex))

def python_iigsexport_pic(img, drawable, filename, raw_filename):
    pdb.gimp_layer_resize_to_image_size(drawable)

    width = drawable.width
    height = drawable.height

    layer = drawable.copy()
    srcRgn = layer.get_pixel_rgn(0, 0, width, height, False, False)
    src_pixels = array("B", srcRgn[0:width, 0:height])
    p_size = len(srcRgn[0,0])

    # print p_size

    dest_pixels = array("B", "\x00" * (width * height * p_size))

    cmbytes, cmdata = pdb.gimp_image_get_colormap(img)

    if (pdb.gimp_drawable_is_indexed(drawable) == 1):
        colorArraySize = cmbytes/3
    else:
        colorArraySize = pdb.plug_in_ccanalyze(img,drawable)

    if (height != 200):
        pdb.gimp_message('Height needs to be 200 pixels.')
        return

    if (width != 320) and (width != 640):
        pdb.gimp_message('Width needs to be 320 or 640 pixels.')
        return

    if (width == 320) and (colorArraySize > 16):
        pdb.gimp_message('Color map needs to be up to 16 colors.')
        return

    if (width == 640) and (colorArraySize > 4):
        pdb.gimp_message('640 mode SHR color map needs to be up to 4 colors.')
        return

    fileOut = open(filename,"wb")


    gimp.progress_init(_("Saving as Apple IIgs Screen Image ($C1/$0000)"))

    # 320 mode
    if width == 320 and (checkDitherPalette(cmdata) == False):

     bytePartCount = 0
     for y in range(0,height):
        for x in range(0,width):
            pixel = src_pixels[(x + width * y) * p_size]

            if bytePartCount == 0:
                value = pixel << 4
            else:
                value += pixel
            bytePartCount+=1
            if bytePartCount == 2:
                fileOut.write(struct.pack("<B", value))
                bytePartCount = 0
            gimp.progress_update(float((x+y*width)*0.8)/(width*height))

    #640 mode -- 4 color
    if (width == 640):

     bytePartCount = 0
     for y in range(0,height):
        for x in range(0,width):
            pixel = src_pixels[(x + width * y) * p_size]
            if bytePartCount == 0:
                value = pixel << 6
            elif bytePartCount == 1:
                value += pixel << 4
            elif bytePartCount == 2:
                value += pixel << 2
            else:
                value += pixel
            bytePartCount+=1
            if bytePartCount == 4:
                fileOut.write(struct.pack("<B", value))
                bytePartCount = 0
            gimp.progress_update(float((x+y*width)*0.8)/(width*height))

    #640 mode -- dither
    if (width == 320) and (checkDitherPalette(cmdata) == True):

     bytePartCount = 0
     for y in range(0,height):
        for x in range(0,width):
            pixel = src_pixels[(x + width * y) * p_size]

            if bytePartCount == 0:
                value = pixel << 4
            else:
                value += pixel
            bytePartCount+=1
            if bytePartCount == 2:
                fileOut.write(struct.pack("<B", value))
                bytePartCount = 0
            gimp.progress_update(float((x+y*width)*0.8)/(width*height))


    # set resolution for each line
    for y in range(0,height):
        value = 0
	if ((width == 640) or ((width == 320) and (checkDitherPalette(cmdata) == True))):
            value = 128
        fileOut.write(struct.pack("<B", value))

    # these are ignored.  set to 0.        
    for y in range(height,256):
        fileOut.write(struct.pack("<B", 0))

    # palette 0 contains the color map
    if ((width == 320) and (checkDitherPalette(cmdata) == False)):
        createPalette320(cmdata,16,fileOut)
    elif ((width == 320) and (checkDitherPalette(cmdata) == True)):
        createPalette640Dither(fileOut)
    else:
        createPalette640(cmdata,4,fileOut)

    # remaining 15 palettes have black for each indexed color
    for y in range(0,480):
        fileOut.write(struct.pack("<B", 0))

    gimp.progress_update(1)

    fileOut.close()

    gimp.delete(img)


def register_save_pic():
    gimp.register_save_handler("file-iigsexportpic-save", "pic", "")

register(
        "file-iigsexportpic-save",
        N_("Save as Apple IIgs SHR color screen PIC $C0/$0000"),
        "Export Apple IIgs SHR Screen PIC",
        "Geoff Weiss",
        "Geoff Weiss",
        "2016",
        N_("Apple IIgs SHR Screen PIC $C0/$0000"),
        "RGB*, GRAY*",
        [
            (PF_IMAGE, "image", "Input image", None),
            (PF_DRAWABLE, "drawable", "Input drawable", None),
            (PF_STRING, "filename", "The name of the file", None),
            (PF_STRING, "raw-filename", "The name of the file", None),
        ],
        [],
        python_iigsexport_pic, on_query=register_save_pic,
        menu="<Save>", domain=("gimp20-python", gimp.locale_directory))


main()

